import test, { describe } from "node:test";
import assert from "node:assert";
import Vault from "../vault.js";
import path from "node:path";

const DOCS_IN_TEST_VAULT = 5;

describe("Vault", () => {
  test("load a vault", () => {
    const vault = new Vault("./bin/_test/test-vault");
    assert.equal(
      vault.size,
      DOCS_IN_TEST_VAULT,
      "Unexpected number of documents in vault",
    );
  });

  test("vault paths are made absolute", () => {
    const vault = new Vault("./bin/_test/test-vault");
    assert.equal(
      path.isAbsolute(vault.vaultPath),
      true,
      "Vault path was not formatted to be absolute",
    );
  });

  test("document tags are properly parsed", () => {
    const vault = new Vault("./bin/_test/test-vault");
    const taggedFile = vault.index["i-have-tags"];

    if (taggedFile === undefined) {
      assert.fail("Expected file with tags");
    }

    const expectedTags = ["propertytag", "tags", "hashtags"];
    assert.deepStrictEqual(taggedFile.tags, expectedTags);
  });

  test("views can generate filtered versions of a vault", () => {
    const vault = new Vault("./bin/_test/test-vault");
    const view = vault.scope((doc) => doc.hasTag("tags"));
    assert.equal(view.size, 1);
  });

  test("processing documents", () => {
    const vault = new Vault("./bin/_test/test-vault");
    vault.process((doc) => doc.setMarkdown(""));

    for (const doc of vault.documents) {
      assert.equal(doc.markdown, "");
    }
  });
});
